//+------------------------------------------------------------------+
//|                                                HighLowLevels.mq4 |
//|          Copyright  2008, RO'Mahoney <evil_choci@yahoo.com>     |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Russ <evil_choci@yahoo.com>"
#property link      ""

#property indicator_chart_window

//---- input parameters
extern color     HighLevelColor	= Red;
extern int       HighLevelWidth	= 4;
extern color     LowLevelColor	= Aqua;
extern int       LowLevelWidth	= 4;
extern int       DaysToControl	= 6;
extern int       ControlTimeFrame= 60;
//extern int       BarsToDraw		= 300;

bool 		Period_Err 	= false;

string	HIGHLEVEL_NAME	= "HL_HighLevel";
string	LOWLEVEL_NAME	= "HL_LowLevel";

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

int init() {
	if ( ControlTimeFrame != 1 && ControlTimeFrame != 5 && ControlTimeFrame != 15 && ControlTimeFrame != 30 && ControlTimeFrame != 60 && ControlTimeFrame != 240 && ControlTimeFrame != 1440 ) {
		Period_Err = true;
		Comment("Control timeframe is incorrect. \nOnly standart periods are possible (1, 5, 15, 30, 60, 240, 1440)");
	}

   return(0);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

int deinit() {
	ObjectDelete(HIGHLEVEL_NAME);
	ObjectDelete(LOWLEVEL_NAME);
	Comment("");
	
   return(0);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

int start() {
	static int nbars = 0;
	int startbar, endbar;
	double hh, ll;
	
	if ( Period_Err ) return(0);
	
	if ( iBars(Symbol(), ControlTimeFrame) != nbars ) {	
		startbar = GetDayStartBar(DaysToControl, ControlTimeFrame);	
		endbar 	= GetDayStartBar(0, ControlTimeFrame);	
	
		hh = GetHighestPrice(startbar, endbar);
		ll = GetLowestPrice(startbar, endbar);
	
		DrawLevels(hh, ll);
		
		nbars = iBars(Symbol(), ControlTimeFrame);
	}
	
   return(0);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

void DrawLevels(double hh, double ll) {
	if ( !(ObjectFind(HIGHLEVEL_NAME) >= 0) ) {
		ObjectCreate(HIGHLEVEL_NAME, OBJ_HLINE, 0, 0, hh);
		ObjectSet(HIGHLEVEL_NAME, OBJPROP_COLOR, HighLevelColor);
		ObjectSet(HIGHLEVEL_NAME, OBJPROP_WIDTH, HighLevelWidth);
	} else {
		ObjectSet(HIGHLEVEL_NAME, OBJPROP_PRICE1, hh);
	}

	if ( !(ObjectFind(LOWLEVEL_NAME) >= 0) ) {
		ObjectCreate(LOWLEVEL_NAME, OBJ_HLINE, 0, 0, ll);
		ObjectSet(LOWLEVEL_NAME, OBJPROP_COLOR, LowLevelColor);
		ObjectSet(LOWLEVEL_NAME, OBJPROP_WIDTH, LowLevelWidth);
	} else {
		ObjectSet(LOWLEVEL_NAME, OBJPROP_PRICE1, ll);
	}

}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

double GetHighestPrice(int startbar, int endbar) {
	double bar = iHighest(Symbol(), ControlTimeFrame, MODE_HIGH, startbar - endbar, endbar);
	double hh = iHigh(Symbol(), ControlTimeFrame, bar);
		
	return(hh);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

double GetLowestPrice(int startbar, int endbar) {
	double bar = iLowest(Symbol(), ControlTimeFrame, MODE_LOW, startbar - endbar, endbar);
	double ll = iLow(Symbol(), ControlTimeFrame, bar);
	
	return(ll);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

int GetDayStartBar(int ndays, int tf) {
	// Receives number of days to loopback and timeframe to search
	// Returns the number of the first bar "ndays" days back
	// Returns 0 if any error
	int i, d = 0;
	
	for ( i = 0; i < Bars - 1; i++ ) {
		if ( TimeDay(iTime(Symbol(), tf, i)) != TimeDay(iTime(Symbol(), tf, i + 1)) ) {
			if ( d == ndays ) return(i);
			else d++;
		}
	}

	return(0);
}