//+------------------------------------------------------------------+
//|                                   Logical Trader Price Pivot.mq4 |
//|                                  Copyright  2010, Steve Hopwood |
//|                              http://www.hopwood3.freeserve.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Steve Hopwood"
#property link      "http://www.hopwood3.freeserve.co.uk"
#include <WinUser32.mqh>
#include <stdlib.mqh>
#define  NL    "\n"
#property show_inputs
#define  startline "Range start line"
#define  endline "Range end line"
#define  rangehighline "Range high line"
#define  rangelowline "Range low line"
#define  bufferhighline "Buffer high line"
#define  bufferlowline "Buffer low line"
#define  pivotline "Pivot line"
#define  pivothighline "Pivot range high line"
#define  pivotlowline "Pivot range low line"


extern string     dr="----Range inputs----";
extern int        RangeShift=0;
extern double     RangeStartTime=8;
extern double     RangeEndTime=16;
extern int        RangeHours=8;
extern color      RangeLineColour=Yellow;
extern int        RangeTimeFrame=0;
extern color      RangeLineHiLoColour=Turquoise;
extern int        PipsBuffer=5;

extern string     pi="----Pivot inputs----";
extern int        PivotTimeFrame = 1440;
extern int        PivotShift = 1;//Can remove the extern when this becomes an indi
extern color      PivotColour=Blue;

extern string     ge="----General inputs----";
extern bool       CriminalHasSundayCandle=true;

//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
datetime hhmm_to_time(double hhmm)
{
  return (StrToTime(DoubleToStr(MathFloor(hhmm),0)+":"+DoubleToStr((hhmm-MathFloor(hhmm))*100,0)) );
}//End datetime hhmm_to_time(double hhmm)

int start()
{


   //Draw range start line
   datetime ConvertedStartTime = hhmm_to_time(RangeStartTime);
   ConvertedStartTime-= 24 * 60 * 60 * RangeShift;
   ObjectDelete(startline);
   ObjectCreate(startline, OBJ_VLINE, 0, ConvertedStartTime, 0);
   ObjectSet(startline, OBJPROP_COLOR, RangeLineColour);
   ObjectSet(startline, OBJPROP_STYLE, STYLE_SOLID);
   
   //Draw range end line
   datetime EndTime = ConvertedStartTime + RangeHours * 3600;
   ObjectDelete(endline);
   ObjectCreate(endline, OBJ_VLINE, 0, EndTime, 0);
   ObjectSet(endline, OBJPROP_COLOR, RangeLineColour);
   ObjectSet(endline, OBJPROP_STYLE, STYLE_DASH);
   
   //Calculate shift. There must be a better way
   double shift;
   for (int cc = 0; cc < iBars(NULL, RangeTimeFrame); cc++)
   {
      if (iTime(NULL, RangeTimeFrame, cc) == ConvertedStartTime) break;
      shift++;      
   }//for (int cc = 0; cc < iBars(null, RangeTimeFrame); cc++)
   
   //Draw range High
   ObjectDelete(rangehighline);
   ObjectCreate(rangehighline,OBJ_TREND,0,ConvertedStartTime,iHigh(NULL, RangeTimeFrame, shift),EndTime,iHigh(NULL, RangeTimeFrame, shift));
   ObjectSet(rangehighline,OBJPROP_RAY,false); 
   ObjectSet(rangehighline,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(rangehighline,OBJPROP_COLOR,RangeLineHiLoColour); 
   
   //Draw range Low
   ObjectDelete(rangelowline);
   ObjectCreate(rangelowline,OBJ_TREND,0,ConvertedStartTime,iLow(NULL, RangeTimeFrame, shift),EndTime,iLow(NULL, RangeTimeFrame, shift));
   ObjectSet(rangelowline,OBJPROP_RAY,false); 
   ObjectSet(rangelowline,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(rangelowline,OBJPROP_COLOR,RangeLineHiLoColour); 
   
   //Adapt PipsBuffer to x digit criminals
   int multiplier;
   if(Digits == 2 || Digits == 4) multiplier = 1;
   if(Digits == 3 || Digits == 5) multiplier = 10;
   if(Digits == 6) multiplier = 100;   
   if(Digits == 7) multiplier = 1000;   
   PipsBuffer*= multiplier;
   
   //Draw upper buffer line
   ObjectDelete(bufferhighline);
   ObjectCreate(bufferhighline,OBJ_TREND,0,ConvertedStartTime,NormalizeDouble(iHigh(NULL, RangeTimeFrame, shift) + (PipsBuffer * Point), Digits),EndTime,NormalizeDouble(iHigh(NULL, RangeTimeFrame, shift) + (PipsBuffer * Point), Digits));
   ObjectSet(bufferhighline,OBJPROP_RAY,false); 
   ObjectSet(bufferhighline,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(bufferhighline,OBJPROP_COLOR,RangeLineHiLoColour); 
   
   //Draw lower buffer line
   ObjectDelete(bufferlowline);
   ObjectCreate(bufferlowline,OBJ_TREND,0,ConvertedStartTime,NormalizeDouble(iLow(NULL, RangeTimeFrame, shift) - (PipsBuffer * Point), Digits),EndTime,NormalizeDouble(iLow(NULL, RangeTimeFrame, shift) - (PipsBuffer * Point), Digits));
   ObjectSet(bufferlowline,OBJPROP_RAY,false); 
   ObjectSet(bufferlowline,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(bufferlowline,OBJPROP_COLOR,RangeLineHiLoColour); 
   

   //Calculate the pivot range
   //Accommodate Sunday candle
   if (CriminalHasSundayCandle && PivotShift == 1)
   {
      int day = (TimeDayOfWeek(TimeLocal()) );
      if (day == 1) PivotShift = 2;
   }//if (CriminalHasSundayCandle && PivotShift == 1)
   
   double high = iHigh(NULL, PivotTimeFrame, PivotShift);
   double low = iLow(NULL, PivotTimeFrame, PivotShift);
   double close = iClose(NULL, PivotTimeFrame, PivotShift);
   
   //Pivot price
   double pp = NormalizeDouble((high + low + close) /3, Digits);
   //Pivot differential
   double pd = MathAbs(pp - (high + low) / 2);
   //Pivot range high
   double prh = NormalizeDouble(pp + pd, Digits);
   //Pivot range low
   double prl = NormalizeDouble(pp - pd, Digits);
   
   //Draw upper pivot line
   ObjectDelete(pivothighline);
   ObjectCreate(pivothighline,OBJ_TREND,0,ConvertedStartTime,prh,EndTime,prh);
   ObjectSet(pivothighline,OBJPROP_RAY,false); 
   ObjectSet(pivothighline,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(pivothighline,OBJPROP_COLOR,PivotColour); 
   
   //Draw pivot line
   ObjectDelete(pivotline);
   ObjectCreate(pivotline,OBJ_TREND,0,ConvertedStartTime,pp,EndTime,pp);
   ObjectSet(pivotline,OBJPROP_RAY,false); 
   ObjectSet(pivotline,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(pivotline,OBJPROP_COLOR,PivotColour); 
   
   
   //Draw pivot line
   ObjectDelete(pivotlowline);
   ObjectCreate(pivotlowline,OBJ_TREND,0,ConvertedStartTime,prl,EndTime,prl);
   ObjectSet(pivotlowline,OBJPROP_RAY,false); 
   ObjectSet(pivotlowline,OBJPROP_STYLE,STYLE_SOLID);
   ObjectSet(pivotlowline,OBJPROP_COLOR,PivotColour); 
   
   
   
   /*
   Alert("Pivot: ", DoubleToStr(pp, Digits), ": Vivot differential: ", DoubleToStr(pd, Digits), 
         ": Range high: ", DoubleToStr(prh, Digits), ": Range low: ", DoubleToStr(prl, Digits),
         ": Close ", DoubleToStr(close, Digits) );
   */

}
//+------------------------------------------------------------------+